/*

    Basic Broadcaster (bb)
    
    For copyright owners see README.txt.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

    For more information about the GNU Public license see:
    http://www.gnu.org/copyleft/gpl.html

*/

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingWorker;
import javax.swing.UIManager;


@SuppressWarnings("serial")
public class BroadcasterGUI extends JFrame implements ConstantsI
{
	private JTextArea txtInput;
	private JTextArea txtOutput;
	private JCheckBox chkOuputHScrollBar;
	private JCheckBox chkInputHScrollBar;
	
	private DatagramSocket sckSender;
	private DatagramSocket sckReceiver;
	private ArrayList<String> ips = new ArrayList<String>();
	
    // Clipboard stuff
    private JCheckBox chkClipboard;
    private boolean copyToClipboard;
    Clipboard systemClipboard;
	
	public BroadcasterGUI()
	{
		super(PROGRAM_NAME);
		super.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		super.setResizable(false);
		
		constructGUI();

		super.setLocationRelativeTo(null);
		super.setVisible(true);
		txtInput.requestFocus();
		
		//ImageIcon icon = new ImageIcon("stock_mail.png");
		//super.setIconImage(icon.getImage());
		
		try {
			sckSender = new DatagramSocket();
			sckReceiver = new DatagramSocket(LISTEN_PORT);
			
			InetAddress[] allMyIps = InetAddress.getAllByName(InetAddress.getLocalHost().getCanonicalHostName());
			for (int i = 0; i < allMyIps.length; i++)
				ips.add(allMyIps[i].getHostAddress());

		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Could not initialize sockets.", "Startup failed", JOptionPane.ERROR_MESSAGE);
			System.exit(-1);
		}
		
		new ListenThread().execute();
	}
	
	private void constructGUI()
	{
		super.setLayout(new BorderLayout());
		super.add(BorderLayout.WEST, Box.createRigidArea(new Dimension(10,10)));
		super.add(BorderLayout.EAST, Box.createRigidArea(new Dimension(10,10)));
		super.add(BorderLayout.NORTH, Box.createRigidArea(new Dimension(10,10)));
		super.add(BorderLayout.SOUTH, Box.createRigidArea(new Dimension(10,10)));
		
		JPanel mainPanel = new JPanel();
		mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
		super.add(BorderLayout.CENTER, mainPanel);
		
		//mainPanel.add(Box.createVerticalStrut(10));
		{
			JPanel subPanel = new JPanel();
			subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.Y_AXIS));
			subPanel.setBorder(BorderFactory.createTitledBorder("output"));
			mainPanel.add(subPanel);
			
			txtOutput = new JTextArea();
			txtOutput.setEditable(false);
			txtOutput.setLineWrap(true);
			JScrollPane sPane = new JScrollPane(txtOutput);
			sPane.setPreferredSize(new Dimension(400, 160));
			sPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			subPanel.add(sPane);
			
			{
				JPanel panel2 = new JPanel();
				panel2.setLayout(new BorderLayout());
				subPanel.add(panel2);
				
				JPanel checkBoxPanel = new JPanel();
				checkBoxPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				panel2.add(BorderLayout.WEST, checkBoxPanel);
				
				chkOuputHScrollBar = new JCheckBox("horizontal scrollbar");
				chkOuputHScrollBar.addItemListener(new ItemListener() {
					@Override
					public void itemStateChanged(ItemEvent e) {
						if ( chkOuputHScrollBar.isSelected() )
							txtOutput.setLineWrap(false);
						else
							txtOutput.setLineWrap(true);
					}
				});
				checkBoxPanel.add(chkOuputHScrollBar);
				
				// Checkbox for clipboard support
                chkClipboard = new JCheckBox("incoming to clipboard");
                chkClipboard.addItemListener(new ItemListener() {
                    @Override
                    public void itemStateChanged(ItemEvent e) {
                        if ( chkClipboard.isSelected() )
                        {
                            copyToClipboard = true;
                            systemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
                        }
                        else
                            copyToClipboard = false;
                    }
                });
                checkBoxPanel.add(chkClipboard);
				
				JPanel buttonPanel = new JPanel();
				buttonPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
				panel2.add(BorderLayout.EAST, buttonPanel);
				
				JButton clearButton = new JButton("X");
				clearButton.addActionListener( new ActionListener() {
					@Override
					public void actionPerformed(ActionEvent arg0) {
						txtOutput.setText(null);
					}
				});
				buttonPanel.add(clearButton);
			}
		}
		mainPanel.add(Box.createVerticalStrut(10));
		{
			JPanel subPanel = new JPanel();
			subPanel.setLayout(new BoxLayout(subPanel, BoxLayout.Y_AXIS));
			subPanel.setBorder(BorderFactory.createTitledBorder("input"));
			mainPanel.add(subPanel);

			txtInput = new JTextArea();
			txtInput.setLineWrap(true);
			JScrollPane sPane = new JScrollPane(txtInput);
			sPane.setPreferredSize(new Dimension(400, 80));
			sPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
			subPanel.add(sPane);
			
			{
				JPanel panel2 = new JPanel();
				panel2.setLayout(new BorderLayout());
				subPanel.add(panel2);
				
				JPanel checkBoxPanel = new JPanel();
				checkBoxPanel.setLayout(new FlowLayout(FlowLayout.LEFT));
				panel2.add(BorderLayout.WEST, checkBoxPanel);
				
				chkInputHScrollBar = new JCheckBox("horizontal scrollbar");
				chkInputHScrollBar.addItemListener(new ItemListener() {
					@Override
					public void itemStateChanged(ItemEvent e) {
						if ( chkInputHScrollBar.isSelected() )
							txtInput.setLineWrap(false);
						else
							txtInput.setLineWrap(true);
					}
				});
				checkBoxPanel.add(chkInputHScrollBar);
			}
		}
		mainPanel.add(Box.createVerticalStrut(10));
		{
			JPanel subPanel = new JPanel();
			subPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 0, 0));
			mainPanel.add(subPanel);
			
			JButton btnSend = new JButton("send!");
			btnSend.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent arg0) {
					try {
						
						sckSender.send( DatagramHelper.createDatagram(txtInput.getText()));
						txtInput.setText(null);
						
					} catch (Exception e) {
						JOptionPane.showMessageDialog(null, "Could not send the message.", "Sending text failed", JOptionPane.ERROR_MESSAGE);
						System.exit(-1);
					} 
				}
			});
			subPanel.add(btnSend);
			super.getRootPane().setDefaultButton(btnSend);
		}
		mainPanel.add(Box.createVerticalStrut(20));
		{
			JPanel subPanel = new JPanel();
			subPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
			mainPanel.add(subPanel);
			
			JPanel labelPanel = new JPanel();
			labelPanel.setLayout( new BorderLayout());
			subPanel.add(labelPanel);
			
			JLabel lbl = new JLabel(PROGRAM_NAME + " v" + PROGRAM_VERSION + " by " );
			labelPanel.add(BorderLayout.NORTH, lbl);
			
			JLabel lbl2 = new JLabel( PROGRAM_AUTHORS );
			labelPanel.add(BorderLayout.SOUTH, lbl2);
		}
		
		super.pack();
	}
	
	
	public static void main(String args[])
	{
        // enable native Look & Feel 
        try
        {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e)
        {
            e.printStackTrace();
        }
		
		new BroadcasterGUI();
	}
	
	public void panick(String caption, String message)
	{
		JOptionPane.showMessageDialog(null, message, caption, JOptionPane.ERROR_MESSAGE);
		System.exit(-1);
	}
	
	
	public class ListenThread extends SwingWorker<Object, Object>
	{
		@SuppressWarnings("deprecation")
		@Override
		protected Object doInBackground()
		{

			try 
			{
				byte[] buf = new byte[MAX_PACKET_LENGTH];
				DatagramPacket packet = new DatagramPacket(buf, buf.length);
				for (;;)
				{
					sckReceiver.receive(packet);
	
					String ip = getSourceIP(packet);
					if ( isLocalIP(ip) )
						continue;
					
					Calendar cal = new GregorianCalendar();
					String date = cal.getTime().toGMTString(); 
					
					txtOutput.append( "(" + date + ") " + ip + ":\n" );
					try
					{
                        String inputText = DatagramHelper.datagramToString(packet);
                        txtOutput.append(inputText);
                        
                        if (copyToClipboard)
                        {
                            // paste to clipboard
                            StringSelection contents = new StringSelection(inputText);
                            systemClipboard.setContents(contents, null);
                        }					    
					}
					catch (Exception e)
					{
						txtOutput.append("-- sry, received garbage... --");
					}
					txtOutput.append( "\n\n" );
					txtOutput.setCaretPosition( txtOutput.getText().length() );
				}
			}
			catch (Exception e)
			{
				panick("Application crashed due to a socket failure.", "Error in receivin thread");
			}
			
			return null;
		}
		
		private String getSourceIP(DatagramPacket packet)
		{
			String s = packet.getSocketAddress().toString();
			s = s.substring(1);
			
			int pos = s.indexOf(":");
			s = s.substring(0, pos);
			
			return s;
		}
		
		private boolean isLocalIP( String ip )
		{
			for (int i = 0; i < ips.size(); i++)
				if ( ips.get(i).equals(ip) )
					return true;
			
			return false;
		}
	};
}
